#include <stdlib.h>
#include <argp.h>
#include "apphi.h"

const char *argp_program_version =
  "APPHi_field 1.0";
const char *argp_program_bug_address =
  "<taos2_apphi@astrosen.unam.mx>";

/* The options we understand. */
static struct argp_option options[] = {
  {"biaslist",  'b', "STRING",      0,
   "File list that include the Bias Files" },
  {"darklist",  'd', "STRING",      0,
   "File list that include the Dark Files" },
  {"flatlist",  'f', "STRING",      0,
   "File list that include the Flat Files" },
  {"sciencelist",   's', "STRING", 0,
  "File list that include the Scientfic Data" },
  {"starlist",  'l', "STRING",      0,
   "File list that include the star position x,y,dim" },
  //{"umbral",   'u', "FLOAT", 0,
  //   "Binarization Umbral to detect stars (0.0 to 1.0)" },
  {"npixels",   'n', "INTEGER", 0,
     "Number of stack to calculate the SNR curve " },
  { 0 }
};


static error_t
parse_opt (int key, char *arg, struct argp_state *state)
{
  struct arguments *arguments = state->input;

  switch (key)
    {
    case 'b':
      arguments->bias = arg;
      break;
    case 'd':
      arguments->dark = arg;
      break;
    case 'f':
      arguments->flat = arg;
      break;
    case 's':
      arguments->science = arg;
      break;
    //case 'u':
    //  arguments->um = atof(arg);
      break;
    case 'n':
      arguments->np = atoi(arg);
      break;
    case 'l':
      arguments->ist =arg;
      break;
    case ARGP_KEY_ARG:
      /* if (state->arg_num >= 1)
       *{
       *  argp_usage(state);
       }*/
      break;
    case ARGP_KEY_END:
      /*if (state->arg_num < 2)
       *{
       *  argp_usage (state);
       }*/
      break;
    default:
      return ARGP_ERR_UNKNOWN;
    }
  return 0;
}

static char doc[] =
  "APPHi_field -- Automated Photometry Pipeline for High Cadence and  Large Volume Data -\va pipeline for  TAOSII Proyect http://taos2.astrosen.unam.mx";

static char args_doc[] = " ";

static struct argp argp = { options, parse_opt, args_doc, doc };

int main (int argc, char **argv)
{
  struct arguments arguments;

  arguments.bias="";
  arguments.flat="";
  arguments.dark="";
  arguments.science="";
  arguments.ist="";

  //argp_usage (state);

  argp_parse (&argp, argc, argv, 0, 0, &arguments);

  printf("\n You arguments are the following:");
  printf("\n Bias list File: %s", arguments.bias);
  printf("\n Flat list File: %s", arguments.flat);
  printf("\n Dark list File: %s", arguments.dark);
  printf("\n Science Data list File: %s", arguments.science);
  printf("\n Stars list File: %s", arguments.ist);
  //printf("\n Umbral: %f", arguments.um);
  printf("\n Number of pixel to generate LC: %d\n", arguments.np);

  apphi_field(arguments);

  exit(0);

}
