/**************************************************************
 *                           APPHI                            *
 * AN AUTOMATED PHOTOMETRY PIPELINE FOR HIGH CADENCE,         *
 *                     LARGE VOLUME DATA                      *
 * Edilberto Sanchez Moreno                                   *
 **************************************************************/
 #include "apphi.h"

 float * get_size_sky(MASTER_DATA * night, int N, int D, long int *naxes)
 {
    long int i;
    float * star;
    naxes[0] = D;
    naxes[1] = D;
    naxes[2] = 0;
    for(i=0 ; i<N ; i++)
    {
        naxes[2] += night[i].naxes[2];
    }
    star = (float*) calloc(naxes[0]*naxes[1]*naxes[2],sizeof(float));
    return star;
 }

float * get_star(int x, int y, MASTER_DATA * night, int N, int D, long int *naxes, float * xy, float * PX, float * PY, float * xyabs,  long int * naxes_sky, float * sky)
{

    long int i, j, k , l, t=0;
    float *star ,*aux;
    long int B = D;
    long int px=0, py=0;
    long int sx=0, sy=0;
    naxes[0] = D;
    naxes[1] = D;
    naxes[2] = 0;
    int Xo = x;
    int Yo = y;

    for(i=0 ; i<N ; i++)
    {
        naxes[2] += night[i].naxes[2];
    }

   star = (float*) calloc(naxes[0]*naxes[1]*naxes[2],sizeof(float));
   aux =  (float*) calloc(B*B,sizeof(float));

        px = x - ((B-1)/2);
        py = y - ((B-1)/2);

   for(k=0 ; k< N ; k++)
   {
        for(l=0 ; l<night[k].naxes[2] ; l++)
        {

        if (px > night[k].naxes[0]-15 || py > night[k].naxes[1]-15 || px < 15 || py < 15)
        {
            for(j=0 ; j<D ; j++)
            {
                for(i=0 ; i<D ; i++)
                {
                star[(l*naxes[1]*naxes[0])+(j*naxes[0])+i] = 1.0 ;
                }
            }
        }
        else
        {
            float max = 0.0;
            float min = 100000.00000;
            px = x - ((B-1)/2);
            py = y - ((B-1)/2);
            for(j=0 ; j<B ; j++)
            {
                for(i=0 ; i<B ; i++)
                {
                    if(night[k].data[(l*night[k].naxes[0]*night[k].naxes[1]) + ((py+j)*night[k].naxes[0]) + (px+i)] > max)
                    max = night[k].data[(l*night[k].naxes[0]*night[k].naxes[1]) + ((py+j)*night[k].naxes[0]) + (px+i)];

                    if(night[k].data[(l*night[k].naxes[0]*night[k].naxes[1]) + ((py+j)*night[k].naxes[0]) + (px+i)] < min)
                    min = night[k].data[(l*night[k].naxes[0]*night[k].naxes[1]) + ((py+j)*night[k].naxes[0]) + (px+i)];
                }
            }

            float P = 1.0;

            for(j=0 ; j<B ; j++)
            {
                for(i=0 ; i<B ; i++)
                {
                    long int pos_v = (l*night[k].naxes[0]*night[k].naxes[1]) + (night[k].naxes[0] * (py+j)) + (px+i);
                    if (night[k].data[pos_v]/max > 0.85)
                    {
                        aux[(j*B) + i] = 1.0;
                        P++;
                    }
                    else
                    {
                        aux[(j*B) + i] = 0.0;
                    }
                }
            }
            if(P > 0 && P < (B*B))
            get_centroids_bin(aux,B,&px,&py);

            if(sqrt(pow(px-x,2)+pow(py-y,2)) < 4)
            {
            x = px;
            y = py;
            }

            xy[t] = sqrt(pow(px-x,2)+pow(py-y,2));
            PX[t] = x;
            PY[t] = y;

            px = x - ((D-1)/2);
            py = y - ((D-1)/2);
            sx = x - ((naxes_sky[0]-1)/2);
            sy = y - ((naxes_sky[1]-1)/2);

            xyabs[t] = sqrt(pow(px-Xo,2)+pow(py-Yo,2));
            t++;

            for(j=0 ; j<B ; j++)
            {
                for(i=0 ; i<B ; i++)
                {

                    long int pos_v = (l*night[k].naxes[0]*night[k].naxes[1]) + (night[k].naxes[0]*(py+j)) + (px+i);
                    long int pos_A = ((j*B) + i) + (l*B*B);
                    star[pos_A] = night[k].data[pos_v];
                }
            }
            for(j=0 ; j<naxes_sky[1] ; j++)
            {
                for(i=0 ; i<naxes_sky[0] ; i++)
                {
                    long int pos_v = (l*night[k].naxes[0]*night[k].naxes[1]) + (night[k].naxes[0] * (sy+j)) + (sx+i);
                    long int pos_A = ((j*naxes_sky[0]) + i) + (l*naxes_sky[0]*naxes_sky[1]);
                    sky[pos_A] = night[k].data[pos_v];
                }
            }
          }
        }
   }
free(aux);
return star;
}

void get_centroids_bin(float *bin, int B, long int *x, long int *y)
{
int i,j;
int vec_x[B];
int vec_y[B];
float mx = 0.0, my = 0.0;
float dx = 0.0;
float dy = 0.0;
for(j=0 ; j<B ; j++)
{
  vec_x[j] = 0;
  vec_y[j] = 0;
    for(i=0 ; i<B ; i++)
    {
      vec_x[j] += bin[(i*B)+j];
      vec_y[j] += bin[(j*B)+i];
    }
}

for(j=0 ; j<B ; j++)
{
   dx += (vec_x[j]*(j));
   dy += (vec_y[j]*(j));
   mx += vec_x[j];
   my += vec_y[j];
}

x[0] = x[0] + (int) (dx/mx);
y[0] = y[0] + (int) (dy/my);
return ;
}

float * get_curve(float * star, float * sky , long int *naxes_star, long int *naxes_sky)
{
float * curve = (float*) calloc(naxes_star[2],sizeof(float));
float A,B;
long int i, j, k,l=0;
int * circle = create_mask_circle(naxes_star[0]);
int * ring = create_mask_ring(naxes_sky[0],naxes_star[0]);

for(k=0 ; k<naxes_star[2]; k++)
{
A = 0.0;
B = 0.0;
    for(i=0 ; i<naxes_star[0] * naxes_star[1]; i++)
    {
    A += star[(k*naxes_star[0] * naxes_star[1])+i] * circle[i];
    }
    for(j=0 ; j<naxes_sky[0] * naxes_sky[1]; j++)
    {
    B += sky[(k*naxes_sky[0] * naxes_sky[0])+j] * ring[j];
    }
curve[k] =  -2.5*log10(A-B);
//curve[k] = -2.5*log10(A);
}

supr_outfl(curve,naxes_star[2]);
 free(circle);
 free(ring);
return curve;
}

void supr_outfl(float *curva, long int naxes)
{
long int i, j;
float p=0.0, s=0.0;
float *aux;
aux =  (float*) calloc(naxes,sizeof(float));
float a,b;

for (i=0; i< naxes ; i++)
 {
   p+= curva[i];
   aux[i] = curva[i];
 }

p = p/(naxes );

for (i=0; i< naxes ; i++)
 {
   s+= pow((curva[i]-p),2) ;
 }
s = sqrt(s/(naxes -1));
a = p - (3.5*s);
b = p + (3.5*s);

for (i=0; i< naxes ; i++)
 {
       if(aux[i] == 0 )
       {
        curva[i] = p;
        aux[i] = p;
       }
 }

for (i=0; i< naxes ; i++)
 {



    if( aux[i] < a || aux[i] > b)
    {
        if(i > 4)
        {
          curva[i] = 0.0;
          for (j=1; j<6 ; j++)
          {
          curva[i]+= aux[i-j];
          }
          curva[i] = curva[i]/5;
        }
        else
        {
          curva[i] = 0.0;
          for (j=1; j<6 ; j++)
          {
          curva[i]+= (aux[i+j]);
          }
          curva[i] = curva[i]/5;
        }
    }

  }
free(aux);
return ;
}

float * get_diff_curve(float * star, float * comp, long int *naxes)
{
long int i;
float *curve;
curve =  (float*) calloc(naxes[2],sizeof(float));
for(i=0 ; i<naxes[2] ; i++)
    {
        curve[i] = (star[i]) - (comp[i]);
    }
return curve;
}

float * copy_vec(float * vec, long int *naxes)
{
long int i;
float *output;
output =  (float*) calloc(naxes[2],sizeof(float));
for(i=0 ; i<naxes[2] ; i++)
    {
    output[i] = vec[i];
    }

return output;
}

long int * size_snr(long int z, long int lz)
{
long int i;
float p, l, s;
long int *t_snr = (long int*) calloc((lz-1),sizeof(long int));
p = log10(z);
l=(6/(float)lz);
s=2;
for(i=0 ; i<(lz-1) ; i++)
  {
    t_snr[i] =(i+1)*10;// (long int)pow(s,p);
    //printf("%ld\n",(int)z/t_snr[i]);
    s= s+l;
  }
return t_snr ;
}

float ** sign_noise(float *star, float *sky, long int *naxes, long int D, long int *z, long int lz)
{
    long int i,j,k,l,p;
    float **curves = (float**) calloc((lz-1),sizeof(float*));
    float *aux_star;
    float *aux_sky;
    long int naxes_sky[3];
    naxes_sky[0] = D;
    naxes_sky[1] = D;
    long int naxes_star[3];
    naxes_star[0] = naxes[0];
    naxes_star[1] = naxes[1];
   for (i=0 ; i<(lz-1) ; i++)
   {
      //printf("%ld\n",i);
       p = (long int)(naxes[2]/z[i]);
       naxes_sky[2] = p;
       naxes_star[2]=p;
       aux_star = (float*) calloc(naxes[0]*naxes[1]*p,sizeof(float));
       aux_sky = (float*) calloc(D*D*p,sizeof(float));
       for (l=0 ; l< p ; l++)
       {
            for (k=0 ; k< z[i] ; k++)
            {
                for (j=0 ; j< (naxes_star[0]*naxes_star[1]) ; j++)
                {
                aux_star[(l*(naxes_star[0]*naxes_star[1]))+j] += star[(l*z[i]*(naxes_star[0]*naxes_star[1]))+(k*(naxes_star[0]*naxes_star[1]))+j];
                }
            }
            for (k=0 ; k< z[i] ; k++)
            {
                for (j=0 ; j< (naxes_sky[0]*naxes_sky[1]) ; j++)
                {
                aux_sky[(l*(naxes_sky[0]*naxes_sky[1]))+j] += sky[(l*z[i]*(naxes_sky[0]*naxes_sky[1]))+(k*(naxes_sky[0]*naxes_sky[1]))+j];
                }
            }

       }
       curves[i] = get_curve(aux_star,aux_sky,naxes_star,naxes_sky);
       free(aux_star);
       free(aux_sky);
   }
return curves;
}

float sign_curve(float *datos, long int z)
{
int j,i,k;
float prom = 0, std = 0, signal;
//printf("Z=%ld\n",z);

for (i=0 ; i<z   ; i++)
{
    prom += datos[i];
}
prom = prom/z;
//printf("prom %f\n",prom);

    for (i=0 ; i<z ; i++)
    {
     //printf("     i=%ld\n",i);
     std += pow(datos[i]-prom, 2);
    }

    std = pow((std/(z-1)),0.5);
    signal = prom/(2.5*std);
    //printf("%f\n",signal);
    return signal;
}

void free_STAR_DATA(STAR_DATA star,long int lz)
{
int i;
for (i=0 ; i<(lz-1) ; i++)
   {
   free(star.C_SNR[i]);
   }
free(star.C_SNR);
free(star.star);
free(star.curve);
free(star.diff);
free(star.z);
free (star.SNR);
free (star.xy);
free (star.xyabs);
free(star.px);
free(star.py);
}
